@@ -63,7 +63,8 @@ sub run {
     foreach my $file ($info->sorted_index) {
         next if not $file->is_file;
         my $fileinfo = $file->file_info;
-        if ($fileinfo =~ m/^[^,]*\bELF\b/ && $fileinfo =~ m/shared object/) {
+        if (   $fileinfo =~ m/^[^,]*\bELF\b/
+            && $fileinfo =~ m/(?:shared object|pie executable)/) {
             my $perm = $file->operm;
             my $debug = defined $objdump->{$file}{DEBUG};
             if ($debug and $perm & 0111 and $file !~ m/\.so(?:\.|$)/) {
@@ -96,7 +97,7 @@ sub run {
             exists $SONAME{$cur_file}
             or (defined $normalized_target
                 and exists $SONAME{$normalized_target})
-          ) {
+        ) {
             # yes!!
             my ($real_file, $perm);
             if (exists $SONAME{$cur_file}) {
@@ -440,6 +441,10 @@ sub run {
             for my $shlib (@shlibs, keys %unversioned_shlibs) {
                 # skip it if it's not a public shared library
                 next unless $ldconfig_dirs->known(dirname($shlib));
+                # Skip Objective C libraries as instance/class methods do not
+                # appear in the symbol table
+                next if any { @{$_}[2] =~ m/^__objc_/ }
+                @{$objdump->{$shlib}{SYMBOLS}};
                 tag 'no-symbols-control-file', $shlib
                   unless is_nss_plugin($shlib);
             }
@@ -453,7 +458,7 @@ sub run {
         my %symbols_control_used;
         my @symbols_depends;
         my $dep_templates = 0;
-        my $meta_info_seen = 0;
+        my %meta_info_seen;
         my $warned = 0;
         my $symbol_count = 0;
 
@@ -492,14 +497,14 @@ sub run {
                 }
 
                 $dep_templates = 0;
-                $meta_info_seen = 0;
                 $symbol_count = 0;
+                undef %meta_info_seen;
             } elsif (m/^\|\s+\S+\s*(?:\(\S+\s+\S+\)|#MINVER#)?/) {
                 # alternative dependency template
 
                 $warned = 0;
 
-                if ($meta_info_seen or not defined $soname) {
+                if (%meta_info_seen or not defined $soname) {
                     tag 'syntax-error-in-symbols-file', $.;
                     $warned = 1;
                 }
@@ -529,7 +534,7 @@ sub run {
                 tag 'syntax-error-in-symbols-file', $.
                   unless defined $soname and $symbol_count == 0;
 
-                $meta_info_seen = 1;
+                $meta_info_seen{$1} = 1;
             } elsif (m/^\s+(\S+)\s(\S+)(?:\s(\S+(?:\s\S+)?))?$/) {
                 # Symbol definition
 
@@ -597,6 +602,8 @@ sub run {
             tag 'unused-shlib-entry-in-symbols-control-file', $shlib_name
               unless $symbols_control_used{$shlib_name};
         }
+        tag 'symbols-file-missing-build-depends-package-field'
+          unless exists $meta_info_seen{'Build-Depends-Package'};
 
         # Check that all of the packages listed as dependencies in the symbols
         # file are satisfied by the current package or its Provides.
